
//=====[ LuҲ]========================================================================================

//=====[FRSKY LuҲ εoe}]=============================================

//=====[M8         JP2(PIN)      JP4 (FRSKY ưw1)]=====
//=====[PD0   	--> 1			--> 1 	      	]===========
//=====[+3.3V 	--> 2 			--> 2  (+3.3V)	]===========
//=====[PD1   	--> 3			--> 3    	  	]===========
//=====[ 						--> 4  (GND)  	]===========
//=====[ 						--> 5  (GND)  	]===========
//=====[						--> 6  (GND)  	]===========
//=====[PD7   	--> 4 			--> 7 	      	]===========
//=====[						--> 8  (GND)  	]===========
//=====[PD6   	--> 5			--> 9	      	]===========
//=====[GND   	--> 6 			--> 10 (GND) 	]===========


//====[o]w]=====================================================
//====[      PD0(1)  PD1(3)  PD7(4) PD6(5)  ]=========
//====[oe:   1        0	   1	  0     ]=========
//====[:   0        1	   0	  1     ]=========

//=====[FRSKY LuҲ CC2500}]=============================================

//=====[M8           JP3(PIN)    JP5(FRSKY ưw2)	]=======
//=====[PB1       --> 1			--> 1  (GDO2)   ]===========
//=====[PB4(MISO) --> 2 	  	--> 2  (SO/GDO1)]===========
//=====[PB5(SCK)  --> 3			--> 3  (SCLK)   ]===========
//=====[PB3(MOSI) --> 4 		--> 4  (SI)     ]===========
//=====[                        --> 5  (GND)    ]===========
//=====[   	                    --> 6  (GND)    ]===========
//=====[PB0       --> 5         --> 7  (GDO0)   ]===========
//=====[PB2(SS)   --> 6 		--> 8  (CSn)    ]===========
//=====[              		 	--> 9  (GND)    ]===========
//=====[GND       --> 7 		--> 10 (GND)    ]===========

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <util/crc16.h>
#include <inttypes.h>
#include <avr/pgmspace.h>

#include <compat/twi.h>

#define MAX_TRIES 50

#define I2C_START 0
#define I2C_DATA 1
#define I2C_DATA_ACK 2
#define I2C_STOP 3
#define ACK 1
#define NACK 0
//======================================================

#define PACKAGE_LEN 9

#define SEL_HI() PORTB |= _BV(PB2)
#define SEL_LO() PORTB &= ~_BV(PB2)

#define SPI_WAIT() while (!(SPSR & (1<< SPIF)))

// CC2500 A
#define CCxxx0_Ready() PINB & _BV(PB4)
#define CRC_OK 0x80
#define BYTES_IN_RXFIFO 0x7F
#define STATE_IDLE 0x01

// Definitions to support burst/single access:
#define WRITE_BURST 0x40
#define READ_SINGLE 0x80
#define READ_BURST 0xC0

// CC2500/CC1100 STROBE, CONTROL AND STATUS REGSITER 
#define CCxxx0_IOCFG2 0x00 // GDO2 output pin configuration
#define CCxxx0_IOCFG1 0x01 // GDO1 output pin configuration
#define CCxxx0_IOCFG0 0x02 // GDO0 output pin configuration
#define CCxxx0_FIFOTHR 0x03 // RX FIFO and TX FIFO thresholds
#define CCxxx0_SYNC1 0x04 // Sync word, high byte
#define CCxxx0_SYNC0 0x05 // Sync word, low byte
#define CCxxx0_PKTLEN 0x06 // Packet length
#define CCxxx0_PKTCTRL1 0x07 // Packet automation control
#define CCxxx0_PKTCTRL0 0x08 // Packet automation control
#define CCxxx0_ADDR 0x09 // Device address
#define CCxxx0_CHANNR 0x0A // Channel number
#define CCxxx0_FSCTRL1 0x0B // Frequency synthesizer control
#define CCxxx0_FSCTRL0 0x0C // Frequency synthesizer control
#define CCxxx0_FREQ2 0x0D // Frequency control word, high byte
#define CCxxx0_FREQ1 0x0E // Frequency control word, middle byte
#define CCxxx0_FREQ0 0x0F // Frequency control word, low byte
#define CCxxx0_MDMCFG4 0x10 // Modem configuration
#define CCxxx0_MDMCFG3 0x11 // Modem configuration
#define CCxxx0_MDMCFG2 0x12 // Modem configuration
#define CCxxx0_MDMCFG1 0x13 // Modem configuration
#define CCxxx0_MDMCFG0 0x14 // Modem configuration
#define CCxxx0_DEVIATN 0x15 // Modem deviation setting
#define CCxxx0_MCSM2 0x16 // Main Radio Control State Machine configuration
#define CCxxx0_MCSM1 0x17 // Main Radio Control State Machine configuration
#define CCxxx0_MCSM0 0x18 // Main Radio Control State Machine configuration
#define CCxxx0_FOCCFG 0x19 // Frequency Offset Compensation configuration
#define CCxxx0_BSCFG 0x1A // Bit Synchronization configuration
#define CCxxx0_AGCCTRL2 0x1B // AGC control
#define CCxxx0_AGCCTRL1 0x1C // AGC control
#define CCxxx0_AGCCTRL0 0x1D // AGC control
#define CCxxx0_WOREVT1 0x1E // High byte Event 0 timeout
#define CCxxx0_WOREVT0 0x1F // Low byte Event 0 timeout
#define CCxxx0_WORCTRL 0x20 // Wake On Radio control
#define CCxxx0_FREND1 0x21 // Front end RX configuration
#define CCxxx0_FREND0 0x22 // Front end TX configuration
#define CCxxx0_FSCAL3 0x23 // Frequency synthesizer calibration
#define CCxxx0_FSCAL2 0x24 // Frequency synthesizer calibration
#define CCxxx0_FSCAL1 0x25 // Frequency synthesizer calibration
#define CCxxx0_FSCAL0 0x26 // Frequency synthesizer calibration
#define CCxxx0_RCCTRL1 0x27 // RC oscillator configuration
#define CCxxx0_RCCTRL0 0x28 // RC oscillator configuration
#define CCxxx0_FSTEST 0x29 // Frequency synthesizer calibration control
#define CCxxx0_PTEST 0x2A // Production test
#define CCxxx0_AGCTEST 0x2B // AGC test
#define CCxxx0_TEST2 0x2C // Various test settings
#define CCxxx0_TEST1 0x2D // Various test settings
#define CCxxx0_TEST0 0x2E // Various test settings

// Strobe commands
#define CCxxx0_SRES 0x30 // Reset chip.
#define CCxxx0_SFSTXON 0x31 // Enable and calibrate frequency synthesizer (if MCSM0.FS_AUTOCAL=1).
                                        // If in RX/TX: Go to a wait state where only the synthesizer is
                                        // running (for quick RX / TX turnaround).
#define CCxxx0_SXOFF 0x32 // Turn off crystal oscillator.
#define CCxxx0_SCAL 0x33 // Calibrate frequency synthesizer and turn it off
                                        // (enables quick start).
#define CCxxx0_SRX 0x34 // Enable RX. Perform calibration first if coming from IDLE and
                                        // MCSM0.FS_AUTOCAL=1.
#define CCxxx0_STX 0x35 // In IDLE state: Enable TX. Perform calibration first if
                                        // MCSM0.FS_AUTOCAL=1. If in RX state and CCA is enabled:
                                        // Only go to TX if channel is clear.
#define CCxxx0_SIDLE 0x36 // Exit RX / TX, turn off frequency synthesizer and exit
                                        // Wake-On-Radio mode if applicable.
#define CCxxx0_SAFC 0x37 // Perform AFC adjustment of the frequency synthesizer
#define CCxxx0_SWOR 0x38 // Start automatic RX polling sequence (Wake-on-Radio)
#define CCxxx0_SPWD 0x39 // Enter power down mode when CSn goes high.
#define CCxxx0_SFRX 0x3A // Flush the RX FIFO buffer.
#define CCxxx0_SFTX 0x3B // Flush the TX FIFO buffer.
#define CCxxx0_SWORRST 0x3C // Reset real time clock.
#define CCxxx0_SNOP 0x3D // No operation. May be used to pad strobe commands to two
                                        // bytes for simpler software.

#define CCxxx0_PARTNUM 0x30
#define CCxxx0_VERSION 0x31
#define CCxxx0_FREQEST 0x32
#define CCxxx0_LQI 0x33
#define CCxxx0_RSSI 0x34
#define CCxxx0_MARCSTATE 0x35
#define CCxxx0_WORTIME1 0x36
#define CCxxx0_WORTIME0 0x37
#define CCxxx0_PKTSTATUS 0x38
#define CCxxx0_VCO_VC_DAC 0x39
#define CCxxx0_TXBYTES 0x3A
#define CCxxx0_RXBYTES 0x3B
#define CCxxx0_RCCTRL1_STATUS 0x3C
#define CCxxx0_RCCTRL0_STATUS 0x3D

#define CCxxx0_PATABLE 0x3E
#define CCxxx0_TXFIFO 0x3F
#define CCxxx0_RXFIFO 0x3F


volatile uint8_t rxBuffer[PACKAGE_LEN];
volatile uint8_t newDataFound;

extern volatile unsigned int VAL_angle;
extern volatile unsigned int VAL_RSSI;
extern volatile unsigned int VAL_LQI;

volatile unsigned int VAL_angle = 0;
volatile unsigned int VAL_LQI = 0;
volatile unsigned int VAL_RSSI = 0;


extern volatile unsigned char i2c_state;
extern volatile unsigned char REG_ID,REG_VH,REG_VL;

volatile unsigned char i2c_state = 0; 
volatile unsigned char REG_ID = 0;
volatile unsigned char REG_VH = 0;
volatile unsigned char REG_VL = 0;

extern volatile unsigned int VAL_ch1,VAL_ch2,VAL_ch3,VAL_ch4,VAL_ch5,VAL_ch6,VAL_ch7,VAL_ch8;

extern volatile unsigned int VAL_buttons;

volatile unsigned int VAL_ch1 = 0;
volatile unsigned int VAL_ch2 = 0;
volatile unsigned int VAL_ch3 = 0;
volatile unsigned int VAL_ch4 = 0;
volatile unsigned int VAL_ch5 = 0;
volatile unsigned int VAL_ch6 = 0;
volatile unsigned int VAL_ch7 = 0;
volatile unsigned int VAL_ch8 = 0;

volatile unsigned int VAL_buttons = 0;




// 8CRCX
uint8_t crc8_ibutton(uint8_t *data, size_t n) {

	uint8_t crc = 0x00;

	while(n) {
		crc = _crc_ibutton_update(crc, *data );
		--n;
	}

	return crc;

}

void SPI_Master_Init() {

	// ]mSS, MOSIMSCKXALJ
	DDRB = _BV(PB1) | _BV(PB2) | _BV(PB3) | _BV(PB5);

	// MISO ]mJAèϯऺWԹq
	PORTB |= _BV(PB4);

	// SSq{԰
	SEL_HI();

	// ϯSPIDҦA]mtvfck/2
	SPCR = _BV(SPE) | _BV(MSTR) | _BV(SPI2X);

	SPSR = 0x00;

}


void halSpiStrobe(uint8_t strobe) {

	SEL_LO();

	while(CCxxx0_Ready());

	SPDR = strobe;
	SPI_WAIT();

	SEL_HI();

}

uint8_t halSpiReadReg(uint8_t addr) {

	uint8_t x;

	SEL_LO();

	while(CCxxx0_Ready());

	SPDR = (addr | READ_SINGLE);
	SPI_WAIT();
	SPDR = 0;
	SPI_WAIT();
	x = SPDR;

	SEL_HI();

	return (x);

}

void halSpiWriteReg(uint8_t addr, uint8_t value) {
    
	SEL_LO();

	while(CCxxx0_Ready());

    SPDR = addr;
	SPI_WAIT();
    SPDR = value;
	SPI_WAIT();

    SEL_HI();

}

void halSpiWriteBurstReg(uint8_t addr, uint8_t *buffer, uint8_t count) {

    uint8_t i;

    SEL_LO();

	while(CCxxx0_Ready());

    SPDR = addr | WRITE_BURST;
    SPI_WAIT();

	for(i= 0;i<count;i++){
		SPDR = buffer[i];
		SPI_WAIT();
    }

    SEL_HI();

}

void halSpiReadBurstReg(uint8_t addr, uint8_t *buffer, uint8_t count) {

    uint8_t i;

    SEL_LO();

	while(CCxxx0_Ready());

    SPDR = (addr | READ_BURST);
    SPI_WAIT();

    for (i = 0; i < count; i++) {
        SPDR = 0;
        SPI_WAIT();
        buffer[i] = SPDR;
    }

    SEL_HI();

}

uint8_t halSpiReadStatus(uint8_t addr) {

    uint8_t x;
    
	SEL_LO();
    
	while(CCxxx0_Ready());

    SPDR = (addr | READ_BURST);
    SPI_WAIT();
    SPDR = 0;
    SPI_WAIT();
    x = SPDR;

    SEL_HI();

    return x;
}

void RESET_CCxxx0() {
        
	SEL_LO();
    
	while(CCxxx0_Ready());

    SPDR = CCxxx0_SRES;
    SPI_WAIT();
    
	SEL_HI();

}

void POWER_UP_RESET_CCxxx0() {

	SEL_HI();
	_delay_ms(1);
	SEL_LO();
	_delay_ms(1);
	SEL_HI();
	_delay_ms(41);

	RESET_CCxxx0();

}

void CC2500_Init() {

	//====[CC2500Wq]=========
	POWER_UP_RESET_CCxxx0();

    //====[]mHs (iý,|sW)]==================
    halSpiWriteReg(CCxxx0_PATABLE, 0xFE);

    halSpiWriteReg(CCxxx0_IOCFG1, 0x07);
    
	halSpiWriteReg(CCxxx0_PKTLEN, PACKAGE_LEN);
    halSpiWriteReg(CCxxx0_PKTCTRL1, 0x84);
	halSpiWriteReg(CCxxx0_PKTCTRL0, 0x0C);
    halSpiWriteReg(CCxxx0_FSCTRL1, 0x09);

    halSpiWriteReg(CCxxx0_FREQ2, 0x5D);
    halSpiWriteReg(CCxxx0_FREQ1, 0xB1);
    halSpiWriteReg(CCxxx0_FREQ0, 0x38);
    
	//====[32Kbps]=============================
	halSpiWriteReg(CCxxx0_MDMCFG4, 0x2A);
    halSpiWriteReg(CCxxx0_MDMCFG3, 0x3B);
    halSpiWriteReg(CCxxx0_MDMCFG2, 0x73);
    halSpiWriteReg(CCxxx0_MDMCFG1, 0xD2);
    halSpiWriteReg(CCxxx0_MDMCFG0, 0xF8);

    halSpiWriteReg(CCxxx0_DEVIATN, 0x01);

    halSpiWriteReg(CCxxx0_MCSM2, 0x07);
    halSpiWriteReg(CCxxx0_MCSM1, 0x30);
	halSpiWriteReg(CCxxx0_MCSM0, 0x18);

    halSpiWriteReg(CCxxx0_FOCCFG, 0x1D);
    halSpiWriteReg(CCxxx0_BSCFG, 0x1C);

    halSpiWriteReg(CCxxx0_AGCCTRL2, 0xC7);
	halSpiWriteReg(CCxxx0_AGCCTRL1, 0x00);
    halSpiWriteReg(CCxxx0_AGCCTRL0, 0xB2);

    halSpiWriteReg(CCxxx0_FREND1, 0xB6);

    halSpiWriteReg(CCxxx0_FSCAL3, 0xCA);
    halSpiWriteReg(CCxxx0_FSCAL0, 0x11);
   
	//====[ե]=======================================
	halSpiStrobe(CCxxx0_SCAL);
	while(halSpiReadStatus(CCxxx0_FSCAL1) == 0x3F);

	//====[M RX FIFO Buffer]========================
	halSpiStrobe(CCxxx0_SFRX);

	//====[iJA]===============================
	halSpiStrobe(CCxxx0_SRX);

}


ISR(TWI_vect)
{

   unsigned char twi_status;

   // Disable Global Interrupt
   cli();

   // Get TWI Status Register, mask the prescaler bits (TWPS1,TWPS0)
   twi_status = TWSR & 0xF8;     

   switch(twi_status) {
     case TW_SR_SLA_ACK:     			 //====[0x60: SLA+W received]=============================================

      	  i2c_state=0; 

       	  TWCR |= (1<<TWINT);   		 //====[^ ACK]===========
          break;

     case TW_SR_DATA_ACK:    	 		 //====[0x80: data received]============================================== 

			switch(i2c_state){
     			case 0:									//====[REG_ID]============================= 
             			REG_ID = TWDR;  
						REG_VH = 0;
						REG_VL = 0;						    
	 		 			i2c_state = 1;

						switch(REG_ID){				
			     			case 0:	
									break;
			     			case 1:									//====[REG_1 Ū(CH1 )]=========
							 		REG_VH = (VAL_ch1 >> 8);  
						 			REG_VL = VAL_ch1; 
									i2c_state = 0;
									break;
			     			case 2:									//====[REG_1 Ū(CH2 )]=========
							 		REG_VH = (VAL_ch2 >> 8);  
						 			REG_VL = VAL_ch2; 
									i2c_state = 0;
									break;
			     			case 3:									//====[REG_1 Ū(CH3 )]=========
							 		REG_VH = (VAL_ch3 >> 8);  
						 			REG_VL = VAL_ch3; 
									i2c_state = 0;
									break;
			     			case 4:									//====[REG_1 Ū(CH4 )]=========
							 		REG_VH = (VAL_ch4 >> 8);  
						 			REG_VL = VAL_ch4; 
									i2c_state = 0;
									break;
			     			case 5:									//====[REG_1 Ū(CH5 )]=========
							 		REG_VH = (VAL_ch5 >> 8);  
						 			REG_VL = VAL_ch5; 
									i2c_state = 0;
									break;
			     			case 6:									//====[REG_1 Ū(CH6 )]=========
							 		REG_VH = (VAL_ch6 >> 8);  
						 			REG_VL = VAL_ch6; 
									i2c_state = 0;
									break;
			     			case 7:									//====[REG_1 Ū(buttons )]=========
							 		REG_VH = (VAL_buttons >> 8);  
						 			REG_VL = VAL_buttons; 
									i2c_state = 0;
									break;
						}

						break;
     			case 1:									//====[REG_VALUE 8줸]=============================
             			REG_VH = TWDR;      
	 		 			i2c_state = 2;
						break;
     			case 2:									//====[REG_VALUE C8줸]=============================
             			REG_VL = TWDR;     
	 		 			i2c_state = 3;

						switch(REG_ID){				
			     			case 0:	
									break;
			     			case 1:									//====[REG_1 ߼g(CH1 )]======= 
//									VAL_ch1 = (REG_VH << 8 );
//									VAL_ch1 = (VAL_ch1 | REG_VL);
									break;
						}
			}

          TWCR |= (1<<TWINT);    		//====[^ ACK]===========
          break;

     case TW_SR_STOP:        			//=====[0xA0: stop or repeated start condition received]===================

 	      if (i2c_state == 3) {
			 i2c_state = 0;	      
       	  }	   

       	  TWCR |= (1<<TWINT);    		//====[^ ACK]===========
          break;

     case TW_ST_SLA_ACK:      			//====[0xA8: SLA+R received]================================================

	 	  TWDR =  REG_VH;      		
	      TWCR |= (1<<TWINT);  			//====[^ ACK]===========
    	  break;

     case TW_ST_DATA_ACK:   			//====[0xB8: data transmitted]==============================================

	 	  TWDR = REG_VL;    	 
	      TWCR |= (1<<TWINT);   		//====[^ ACK]===========
    	  break;

     case TW_ST_DATA_NACK:   			//====[0xC0: data transmitted, NACK received]===============================
     case TW_ST_LAST_DATA:   			//====[0xC8: last data byte transmitted, ACK received]======================
     case TW_BUS_ERROR:      			//====[0x00: illegal start or stop condition]===============================
     default:
       TWCR |= (1<<TWINT);    			//====[^ ACK]===========
       i2c_state = 0;         
   }

   // Enable Global Interrupt
   sei();
}


int main(void)
{

	uint8_t i, old_Count;


	DDRC = 0xFF;
	PORTC = 0x00;

	DDRD = 0xFF;								//====[X]==========  
//	PORTD = _BV(PD0) | _BV(PD7);				//====[]wLuҲլoe]========== 
	PORTD = _BV(PD1) | _BV(PD6);				//====[]wLuҲլ]==========

 	//====[Initial I2C Slave]===================================================================================
  	TWAR = (0x11 << 1); 	   //====[]wqa} 0x11]=======
  	TWDR = 0x00;               //====[ks ]===================
  	TWBR = 0x30;

	//====[Start Slave Listening: Clear TWINT Flag, Enable ACK, Enable TWI, TWI Interrupt Enable]=========
  	TWCR = (1<<TWINT) | (1<<TWEA) | (1<<TWEN) | (1<<TWIE);

	//====[ɰ_(iٲ)]========
//	_delay_ms(1000);


    i = 0;
    while(--i)
        _delay_ms(2);

	// lCC2500
	SPI_Master_Init();
	CC2500_Init();

	newDataFound = 0;
	old_Count = 0xFF;

	sei();

	while(1){

		//====[˴ CC2500 O_ƾڥ]]=======================================================

		if (halSpiReadStatus(CCxxx0_MARCSTATE) == STATE_IDLE ) {

			//=====[ƾ]=============================================================
			halSpiReadBurstReg(CCxxx0_RXFIFO, (uint8_t *)rxBuffer, PACKAGE_LEN);

//			if (halSpiReadStatus(CCxxx0_PKTSTATUS) & CRC_OK) {
//				if (rxBuffer[PACKAGE_LEN - 1] != old_Count) {
//					old_Count = rxBuffer[PACKAGE_LEN - 1];
//					newDataFound = 1;
//				}
//			}

//	    	VAL_LQI  = halSpiReadReg(0x33);


			VAL_ch1 = ((rxBuffer[0] & 0xFF) << 2) | (rxBuffer[1] >> 6);
			VAL_ch2 = ((rxBuffer[1] & 0x3F) << 4) | (rxBuffer[2] >> 4);
			VAL_ch3 = ((rxBuffer[2] & 0x0F) << 6) | (rxBuffer[3] >> 2);
			VAL_ch4 = ((rxBuffer[3] & 0x03) << 8) | rxBuffer[4];
			VAL_ch5 = ((rxBuffer[5] & 0xFF) << 2) | (rxBuffer[6] >> 6);
			VAL_ch6 = ((rxBuffer[6] & 0x3F) << 4) | (rxBuffer[7] >> 4);
			VAL_buttons = (rxBuffer[7] & 0x0F) | (rxBuffer[8] << 4);

			//=====[MűFIFO Buffer]==============
			halSpiStrobe(CCxxx0_SFRX);

			//=====[iJA]=====================
			halSpiStrobe(CCxxx0_SRX);

		} 



//			PORTC = ~PORTC;			//====[25usg]===========

        _delay_ms(10);

    }

    return 0;
}
