

//====[DISPLAY TWI qa} 0x01 ]============================================== 
//====[LCD 16x02 ܼҲX]=================

//#define F_CPU 12000000UL  // 12 MHz

#include <avr/io.h>
#include <util/delay.h>
#include <avr\interrupt.h>
#include <compat/twi.h>

extern volatile unsigned char REG_ID,REG_VH,REG_VL;

volatile unsigned char REG_ID = 0;
volatile unsigned char REG_VH = 0;
volatile unsigned char REG_VL = 0;

extern volatile unsigned int VAL_ch1,VAL_ch2,VAL_ch3,VAL_ch4,VAL_ch5,VAL_ch6;
extern volatile unsigned int VAL_buttons;

volatile unsigned int VAL_ch1 = 0;
volatile unsigned int VAL_ch2 = 0;
volatile unsigned int VAL_ch3 = 0;
volatile unsigned int VAL_ch4 = 0;
volatile unsigned int VAL_ch5 = 0;
volatile unsigned int VAL_ch6 = 0;

volatile unsigned int VAL_buttons = 0;


const unsigned char DisTab[]={
	0x3F,		//====[0]====
	0x06,		//====[1]====
	0x5B,		//====[2]====
	0x4F,		//====[3]====
	0x66,		//====[4]====
	0x6D,		//====[5]====
	0x7D,		//====[6]====
	0x07,		//====[7]====
	0x7F,		//====[8]====
	0x6F,		//====[9]====
	0x77,		//====[A]====
	0x7C,		//====[b]====
	0x39,		//====[C]====
	0x5E,		//====[d]====
	0x79,		//====[E]====
	0x71,		//====[F]====
	0x80,		//====[.]====
	};

#define LCDPORT PORTD
#define LCDDDR DDRD
#define LCDPIN PIND

#define En_H PORTB |= _BV(PB2) 		//En----PB2
#define En_L PORTB &= ~_BV(PB2) 

#define RW_R PORTB |= _BV(PB1) 		//RW----PB1
#define RW_W PORTB &= ~_BV(PB1)

#define RS_H PORTB |= _BV(PB0) 		//RS----PB0
#define RS_L PORTB &= ~_BV(PB0) 

#define DelaytE _delay_us(2)

//#define Clear_Screen Write_Command(0x01)


void Delay(uint16_t time)
{
	while(time>0)
	{
		time--;
	}
}


void En_Toggle(void)
{
	En_H; 
	DelaytE; 
	En_L; 
	DelaytE;
}

void Wait_Until_Ready(void)
{
	LCDDDR = 0x00;

	RS_L; 
	RW_R;

	DelaytE;
	En_H;
	DelaytE;

	loop_until_bit_is_clear(LCDPIN,7);

	En_L;
}

void Write_Command(uint8_t Command,uint8_t busyFlag)
{
	uint8_t busy;
	busy = busyFlag;

	if (busy == 1) Wait_Until_Ready();

	LCDDDR = 0xff;
	RS_L; 
	RW_W; 
	LCDPORT = Command;

	_delay_us(25);

	En_Toggle(); 
}

void Write_Data(uint8_t data)
{
	Wait_Until_Ready(); 

	RS_H;
	RW_W; 

	LCDDDR = 0xff;
	LCDPORT = data;

	En_Toggle(); 
}

void Initialize_LCD(void)
{
	LCDDDR = 0xff; 
	DDRB   = 0xff; 
	
	//====[]========================	
	_delay_ms(15);
	Write_Command(0x38,0);
	_delay_ms(5);
	Write_Command(0x38,0);
	_delay_ms(5);
	Write_Command(0x38,0);

	//====[]===============================
	Write_Command(0x3C,1);
	Write_Command(0x08,1); 
	Write_Command(0x01,1); 
	Write_Command(0x06,1); 
	Write_Command(0x0f,1); 
} 


ISR(TWI_vect)
{
	static unsigned char i2c_state;
	unsigned char twi_status;

   // Disable Global Interrupt
   cli();

   // Get TWI Status Register, mask the prescaler bits (TWPS1,TWPS0)
   twi_status = TWSR & 0xF8;     

   switch(twi_status) {
     case TW_SR_SLA_ACK:     	 // 0x60: SLA+W received, ACK returned

      	  i2c_state=0;           // Start I2C State for Register Address required	 

       	  TWCR |= (1<<TWINT);    // Clear TWINT Flag
          break;

     case TW_SR_DATA_ACK:    	 // 0x80: data received, ACK returned

			switch(i2c_state){				
     			case 0:									//====[REG_ID]=============================
             			REG_ID = TWDR;  
						REG_VH = 0;
						REG_VL = 0;						    
	 		 			i2c_state = 1;
						break;
     			case 1:									//====[REG_VALUE 8줸]=============================
             			REG_VH = TWDR;      
	 		 			i2c_state = 2;
						break;
     			case 2:									//====[REG_VALUE C8줸]=============================
             			REG_VL = TWDR;     
	 		 			i2c_state = 3;

						switch(REG_ID){				
			     			case 0:	
									break;
			     			case 1:										//====[ch1]=======
									VAL_ch1 = (REG_VH << 8 );
									VAL_ch1 = (VAL_ch1 | REG_VL);
									break;
			     			case 2:										//====[ch2]=======
									VAL_ch2 = (REG_VH << 8 );
									VAL_ch2 = (VAL_ch2 | REG_VL);
									break;
			     			case 3:										//====[ch3]=======	
									VAL_ch3 = (REG_VH << 8 );
									VAL_ch3 = (VAL_ch3 | REG_VL);
									break;
			     			case 4:										//====[ch4]=======
									VAL_ch4 = (REG_VH << 8 );
									VAL_ch4 = (VAL_ch4 | REG_VL);
									break;
			     			case 5:										//====[ch5]=======
									VAL_ch5 = (REG_VH << 8 );
									VAL_ch5 = (VAL_ch5 | REG_VL);
									break;
			     			case 6:										//====[ch6]=======
									VAL_ch6 = (REG_VH << 8 );
									VAL_ch6 = (VAL_ch6 | REG_VL);
									break;
			     			case 7:										//====[buttons }]=======
									VAL_buttons = (REG_VH << 8 );
									VAL_buttons = (VAL_buttons | REG_VL);
									break;
						}

						break;
  			}
 

          TWCR |= (1<<TWINT);    // Clear TWINT Flag
          break;

     case TW_SR_STOP:        	 // 0xA0: stop or repeated start condition received while selected

    		TWCR |= (1<<TWINT);    // Clear TWINT Flag

			if (i2c_state == 3) {
				i2c_state = 0;
			}	   
 
        	break;

     case TW_ST_DATA_NACK:    // 0xC0: data transmitted, NACK received
     case TW_ST_LAST_DATA:    // 0xC8: last data byte transmitted, ACK received
     case TW_BUS_ERROR:       // 0x00: illegal start or stop condition
     default:
		  	TWCR |= (1<<TWINT);    // Clear TWINT Flag
			i2c_state = 0;         // Back to the Begining State
   }

   // Enable Global Interrupt
   sei();
}

int main(void)
 {

	unsigned char c = 0;
	unsigned char i = 0;
	unsigned char j = 0;
	unsigned char chbuf[5] = {0,0,0,0,0};

	unsigned int a,temp;

	MCUCR  = 0x00;
	MCUCSR = 0x80;
	GICR   = 0x00;

 	//====[Initial I2C Slave]===================================================================================
  	TWAR = (0x01 << 1); 	   //====[]wqa} 0x01]=======
  	TWDR = 0x00;               //====[ks ]===============
  	TWBR = 0x30;

	//====[Start Slave Listening: Clear TWINT Flag, Enable ACK, Enable TWI, TWI Interrupt Enable]=========
  	TWCR = (1<<TWINT) | (1<<TWEA) | (1<<TWEN) | (1<<TWIE);

	_delay_ms(1000);

	sei();

	Initialize_LCD();

	a = 0;

 	while(1){

		//====[DISLPAY VALUE 1 (ch1)]=======================================================================

		a = VAL_ch1;
		
		temp = a;

		c = 0;
		i = 0;
		j = 0;

		while (i < 5)
		{
			//====[]================================
			chbuf[j] = temp - ((temp/10)*10);
			temp = temp/10;
			j++;
			i++;
		}

		//=====[]wЦm Y=0,X=0]=========================		
		c = 0x80;
		Write_Command(c,1);
		
		//=====[]wܦr (Ʀ 5)]=========================
//		c = chbuf[4] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 4)]=========================
//		c = chbuf[3] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 3)]=========================
		c = chbuf[2] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 2)]=========================
		c = chbuf[1] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 1)]=========================
		c = chbuf[0] + 0x30;
		Write_Data(c);


		//====[DISLPAY VALUE 2 (ch2)]==============================================================

		a = VAL_ch2;	
		temp = a;

		c = 0;
		i = 0;
		j = 0;

		while (i < 5)
		{
			//====[]================================
			chbuf[j] = temp - ((temp/10)*10);
			temp = temp/10;
			j++;
			i++;
		}

		//=====[]wܦr (Ů)]===========================
		c = 0x20;
		Write_Data(c);
			
		//=====[]wܦr (Ʀ 5)]=========================
//		c = chbuf[4] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 4)]=========================
//		c = chbuf[3] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 3)]=========================
		c = chbuf[2] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 2)]=========================
		c = chbuf[1] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 1)]=========================
		c = chbuf[0] + 0x30;
		Write_Data(c);

		//====[DISLPAY VALUE 3 (ch3)]==============================================================

		a = VAL_ch3;	
		temp = a;

		c = 0;
		i = 0;
		j = 0;

		while (i < 5)
		{
			//====[]================================
			chbuf[j] = temp - ((temp/10)*10);
			temp = temp/10;
			j++;
			i++;
		}

		//=====[]wܦr (Ů)]===========================
		c = 0x20;
		Write_Data(c);
			
		//=====[]wܦr (Ʀ 5)]=========================
//		c = chbuf[4] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 4)]=========================
//		c = chbuf[3] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 3)]=========================
		c = chbuf[2] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 2)]=========================
		c = chbuf[1] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 1)]=========================
		c = chbuf[0] + 0x30;
		Write_Data(c);


		//====[DISLPAY VALUE 4 (ch4)]===============================

		a = VAL_ch4;
		
		temp = a;

		c = 0;
		i = 0;
		j = 0;

		while (i < 5)
		{
			//====[]================================
			chbuf[j] = temp - ((temp/10)*10);
			temp = temp/10;
			j++;
			i++;
		}

		//=====[]wܦr (Ů)]===========================
		c = 0x20;
		Write_Data(c);
		
		//=====[]wܦr (Ʀ 5)]=========================
//		c = chbuf[4] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 4)]=========================
//		c = chbuf[3] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 3)]=========================
		c = chbuf[2] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 2)]=========================
		c = chbuf[1] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 1)]=========================
		c = chbuf[0] + 0x30;
		Write_Data(c);


		//====[DISLPAY VALUE 5 (ch5)]==============================================================

		a = VAL_ch5;	
		temp = a;

		c = 0;
		i = 0;
		j = 0;

		while (i < 5)
		{
			//====[]================================
			chbuf[j] = temp - ((temp/10)*10);
			temp = temp/10;
			j++;
			i++;
		}

		//=====[]wЦm Y=0,X=0]=========================		
		c = 0xC0;
		Write_Command(c,1);
					
		//=====[]wܦr (Ʀ 5)]=========================
//		c = chbuf[4] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 4)]=========================
//		c = chbuf[3] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 3)]=========================
		c = chbuf[2] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 2)]=========================
		c = chbuf[1] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 1)]=========================
		c = chbuf[0] + 0x30;
		Write_Data(c);

		//====[DISLPAY VALUE 6 (ch6)]==============================================================

		a = VAL_ch6;	
		temp = a;

		c = 0;
		i = 0;
		j = 0;

		while (i < 5)
		{
			//====[]================================
			chbuf[j] = temp - ((temp/10)*10);
			temp = temp/10;
			j++;
			i++;
		}

		//=====[]wܦr (Ů)]===========================
		c = 0x20;
		Write_Data(c);
			
		//=====[]wܦr (Ʀ 5)]=========================
//		c = chbuf[4] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 4)]=========================
//		c = chbuf[3] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 3)]=========================
		c = chbuf[2] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 2)]=========================
		c = chbuf[1] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 1)]=========================
		c = chbuf[0] + 0x30;
		Write_Data(c);


		//====[DISLPAY VALUE 7 ()]==============================================================
		c = 0x20;
		Write_Data(c);
		c = 0x20;
		Write_Data(c);
		c = 0x20;
		Write_Data(c);
		c = 0x20;
		Write_Data(c);
						
						
						
		//====[DISLPAY VALUE 8 (buttons)]==============================================================

		a = VAL_buttons;	
		temp = a;

		c = 0;
		i = 0;
		j = 0;

		while (i < 5)
		{
			//====[]================================
			chbuf[j] = temp - ((temp/10)*10);
			temp = temp/10;
			j++;
			i++;
		}

		//=====[]wܦr (Ů)]===========================
		c = 0x20;
		Write_Data(c);
			
		//=====[]wܦr (Ʀ 5)]=========================
//		c = chbuf[4] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 4)]=========================
//		c = chbuf[3] + 0x30;
//		Write_Data(c);
		//=====[]wܦr (Ʀ 3)]=========================
		c = chbuf[2] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 2)]=========================
		c = chbuf[1] + 0x30;
		Write_Data(c);
		//=====[]wܦr (Ʀ 1)]=========================
		c = chbuf[0] + 0x30;
		Write_Data(c);

		//====[]================================================================================
		_delay_ms(20);

	} 

}
